@@ -20,8 +20,8 @@
  * @file Sam Meder, Sam Lang
  *
  * $RCSfile: globus_gsi_proxy_handle.c,v $
- * $Revision: 1.21 $
- * $Date: 2008/09/15 15:06:26 $
+ * $Revision: 1.21.4.1 $
+ * $Date: 2010/06/28 19:17:34 $
  */
 
 #define GLOBUS_GSI_PROXY_HANDLE_MALLOC_ERROR(_LENGTH_) \
@@ -646,19 +646,19 @@ globus_gsi_proxy_handle_set_type(
       case GLOBUS_GSI_CERT_UTILS_TYPE_GSI_3_IMPERSONATION_PROXY:
       case GLOBUS_GSI_CERT_UTILS_TYPE_RFC_IMPERSONATION_PROXY:
         result = globus_gsi_proxy_handle_set_policy(
-            handle, NULL, 0, OBJ_sn2nid(IMPERSONATION_PROXY_SN));
+            handle, NULL, 0, OBJ_txt2nid(IMPERSONATION_PROXY_OID));
         break;
 
       case GLOBUS_GSI_CERT_UTILS_TYPE_GSI_3_INDEPENDENT_PROXY:
       case GLOBUS_GSI_CERT_UTILS_TYPE_RFC_INDEPENDENT_PROXY:
         result = globus_gsi_proxy_handle_set_policy(
-            handle, NULL, 0, OBJ_sn2nid(INDEPENDENT_PROXY_SN));
+            handle, NULL, 0, OBJ_txt2nid(INDEPENDENT_PROXY_OID));
         break;
 
       case GLOBUS_GSI_CERT_UTILS_TYPE_GSI_3_LIMITED_PROXY:
       case GLOBUS_GSI_CERT_UTILS_TYPE_RFC_LIMITED_PROXY:
         result = globus_gsi_proxy_handle_set_policy(
-            handle, NULL, 0, OBJ_sn2nid(LIMITED_PROXY_SN));
+            handle, NULL, 0, OBJ_txt2nid(LIMITED_PROXY_OID));
         break;
       default:
         break;
